#ifndef __TSI_LIB_CONF_H__
#define __TSI_LIB_CONF_H__

#include "tsi_def.h"

/* Chip info ----------------------------------------------------------------*/
/* Chip total channel num */
#define TSI_TOTAL_CHANNEL_NUM           (14U)

/* Resource usage -----------------------------------------------------------*/
/* Widget num */
#define TSI_TOTAL_WIDGETS_NUM           (3U)

/* Widget usage(0 - not used, 1 - used) */
#define TSI_WIDGET_BUTTON_USED          (0U)    /* Button */
#define TSI_WIDGET_PROXIMITY_USED       (0U)    /* Proximity */
#define TSI_WIDGET_SLIDER_USED          (0U)    /* Slider */

/* Total sensor num */
#define TSI_TOTAL_SENSORS_NUM           (7U)

/* Sensor num per widget */
#define TSI_MODE_SENSORS_NUM         (1U)
#define TSI_SLIDER_SENSORS_NUM         (5U)
#define TSI_POWER_SENSORS_NUM         (1U)

/* Total pin num */
#define TSI_TOTAL_PINS_NUM              (7U)

/* TSI module features ------------------------------------------------------*/
/* Use compensation IDAC(0 - not used, 1 - used) */
#define TSI_USE_COMP_IDAC               (1U)

/* Use shield(0 - not used, 1 - used) */
#define TSI_USE_SHIELD                  (0U)

/* Use DMA(0 - not used, 1 - used) */
#define TSI_USE_DMA                     (0U)

/* Operation clock configurations -------------------------------------------*/
/* Sample clock source */
#define TSI_SAMPLE_CLK_SOURCE           (TSI_SAMPLE_CLK_PRS)

/* Algorithm configurations -------------------------------------------------*/
/* Enable/disable regular channel filter */
#define TSI_REGULAR_FILTER_EN               (1U)

/* Enable/disable regular channel average filter(4-sample) */
#define TSI_REGULAR_FILTER_AVERAGE_EN       (1U)
#define TSI_REGULAR_FILTER_AVERAGE_ORDER    (3U)

/* Enable/disable regular channel median filter(3-sample) */
#define TSI_REGULAR_FILTER_MEDIAN_EN        (1U)
#define TSI_REGULAR_FILTER_MEDIAN_ORDER     (2U)

/* Enable/disable regular channel first order IIR filter */
#define TSI_REGULAR_FILTER_IIR_EN           (0U)
#define TSI_REGULAR_FILTER_IIR_COEF         (1U)

/* Enable/disable proximity channel filter */
#define TSI_PROX_FILTER_EN                  (0U)

/* Enable/disable proximity channel average filter(4-sample) */
#define TSI_PROX_FILTER_AVERAGE_EN          (0U)
#define TSI_PROX_FILTER_AVERAGE_ORDER       (3U)

/* Enable/disable proximity channel median filter(3-sample) */
#define TSI_PROX_FILTER_MEDIAN_EN           (0U)
#define TSI_PROX_FILTER_MEDIAN_ORDER        (2U)

/* Enable/disable proximity channel advanced 2-stage IIR filter */
#define TSI_PROX_FILTER_ADVIIR_EN           (0U)

#define TSI_PROX_FILTER_ADVIIR_COEFF_PERF_QP    \
static const uint32_t ADVIIR_COEFF_PERF_QP = 30U;
    
#define TSI_PROX_FILTER_ADVIIR_COEFF_PERF    \
static const int32_t ADVIIR_COEFF_PERF[2][4] = {   \
    {16761928, 33523857, 16761928, 0},          \
    {1073741824, -1735991514, 729297403, 0},    \
};

#define TSI_PROX_FILTER_ADVIIR_COEFF_FAST_QP    \
static const uint32_t ADVIIR_COEFF_FAST_QP = 30U;

#define TSI_PROX_FILTER_ADVIIR_COEFF_FAST    \
static const int32_t ADVIIR_COEFF_FAST[2][4] = {    \
    {171812690, 515438070, 515438070, 171812690},   \
    {1073741824, -53043456, 358606952, -4824679},   \
};

/* Baseline algorithm configurations ----------------------------------------*/
/* Baseline algorithm type */
#define TSI_SENSOR_BASELINE_TYPE            (TSI_SENSOR_IIR_BASELINE)

/* Sensor auto reset */
#define TSI_SENSOR_AUTO_RESET_EN            (0U)

/* Misc configurations ------------------------------------------------------*/
/* Startup initial scan timeout */
#define TSI_STARTUP_TIMEOUT_VALUE           (0xFFFFFFFFU)

/* Calibration scan timeout */
#define TSI_CALIB_TIMEOUT_VALUE             (0xFFFFFFFFU)

/* Calibration method */
#define TSI_CALIB_METHOD                    (TSI_CALIB_METHOD_HARDWARE)

/* Inactive sensor connection */
#define TSI_INACTIVE_SENSOR_CONNECTION      (TSI_INACTIVE_SENSOR_GROUNDED)

/* TSI module interrupt priority */
#define TSI_INTERRUPT_PRIORITY              (3U)

/* Slider sensor position resolution */
#define TSI_SLIDER_RESOLUTION               (255U)

/* Calibration constants ----------------------------------------------------*/
#define TSI_CALIB_TARGET_PERCENTAGE         (85U)
#define TSI_IDAC_BITWIDTH                   (7U)     
#define TSI_CALIB_INITIAL_RESOLUTION        (13U)
#define TSI_MAX_SMPL_CLK_DIV                (32U)
#define TSI_MAX_SMPL_CLK                    (3740000U)  
#define TSI_MAX_CS                          (50U)
#define TSI_MIN_CS                          (5U)
#define TSI_MAX_RESOLUTION                  (16U)
#define TSI_MIN_RESOLUTION                  (8U)
#define TSI_SERIES_RESISTOR_TOTAL           (1060U)

/* Widget and sensor list definition ----------------------------------------*/
#define TSI_WIDGET_LIST_DEFINE  \
typedef struct _TSI_WidgetListStruct   \
{   \
    TSI_ButtonWidgetTypeDef mode;	\
    TSI_SliderWidgetTypeDef slider;	\
    TSI_ButtonWidgetTypeDef power;	\
} TSI_WidgetListTypeDef;

#define TSI_SENSOR_LIST_DEFINE  \
typedef struct _TSI_SensorListStruct   \
{   \
    TSI_SensorTypeDef mode[TSI_MODE_SENSORS_NUM];	\
    TSI_SensorTypeDef slider[TSI_SLIDER_SENSORS_NUM];	\
    TSI_SensorTypeDef power[TSI_POWER_SENSORS_NUM];	\
} TSI_SensorListTypeDef;

#endif  /* __TSI_LIB_CONF_H__ */



