#include "tsi_objects.h"
#include <string.h>

/* Objects ----------------------------------------------------------------------*/
TSI_ALIGN4 TSI_USED TSI_ModuleTypeDef TSI_Module TSI_MODULE_SECTION;
TSI_ALIGN4 TSI_USED TSI_WidgetListTypeDef TSI_WidgetList TSI_WIDGETS_SECTION;
TSI_ALIGN4 TSI_USED TSI_SensorListTypeDef TSI_SensorList TSI_SENSORS_SECTION;

/* Filter buffers ---------------------------------------------------------------*/
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterMode[TSI_MODE_SENSORS_NUM];
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterSlider[TSI_SLIDER_SENSORS_NUM];
static TSI_USED TSI_RegularSnsFilterTypeDef TSI_FilterPower[TSI_POWER_SENSORS_NUM];

/* Debounce buffers -------------------------------------------------------------*/
static TSI_USED uint8_t TSI_DebounceMode[TSI_MODE_SENSORS_NUM];
static TSI_USED uint8_t TSI_DebounceSlider[TSI_SLIDER_SENSORS_NUM];
static TSI_USED uint8_t TSI_DebouncePower[TSI_POWER_SENSORS_NUM];

/* Initial values ---------------------------------------------------------------*/
/* TSI Module */
TSI_ALIGN4 TSI_USED const TSI_ModuleTypeDef TSI_ModuleConstInit TSI_CONST_MODULE_SECTION = 
{
    453880014,  /* firmwareId */
    0,  /* debugCmd */
    /* hardwareCfg */
    {
        0,  /* opClkSource */
        0,  /* opClkPrescaler */
        24000000,   /* opClkFreq */
    },
    /* scanCfg */
    {
        0,  /* scanPeriod */
        0,  /* sampleInterval */
        0,  /* sampleIdleTime */
    },
    TSI_LIB_RESET,  /* status */
};

/* TSI sensor list */
TSI_ALIGN4 TSI_USED const TSI_SensorListTypeDef TSI_SensorListConstInit TSI_CONST_SENSORS_SECTION = 
{
    {   
        {   /* Mode_Sns0 */
            &TSI_MetaSensors[0],
            0U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
    {   
        {   /* Slider_Sns0 */
            &TSI_MetaSensors[1],
            0U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
        {   /* Slider_Sns1 */
            &TSI_MetaSensors[2],
            0U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
        {   /* Slider_Sns2 */
            &TSI_MetaSensors[3],
            0U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
        {   /* Slider_Sns3 */
            &TSI_MetaSensors[4],
            0U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
        {   /* Slider_Sns4 */
            &TSI_MetaSensors[5],
            0U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
    {   
        {   /* Power_Sns0 */
            &TSI_MetaSensors[6],
            0U,	/* idacComp */
            0,	/* rawCount */
            0,	/* baseline */
            0,	/* diffCount */
            0,	/* iirBuffer */
            0,	/* padding */
            0,	/* status */
        },
    },
};

/* TSI widget list */
TSI_ALIGN4 TSI_USED const TSI_WidgetListTypeDef TSI_WidgetListConstInit TSI_CONST_WIDGETS_SECTION = 
{
    {   /* Mode */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[0],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            8U,	/* resolution */
            1U,	/* sampleClkDiv */
            0U,	/* idacMod */
            40U,	/* activeTh */
            10U,	/* activeHys */
            10U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            100U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
        0U,	/* buttonStatus */
    },
    {   /* Slider */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[1],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            8U,	/* resolution */
            1U,	/* sampleClkDiv */
            0U,	/* idacMod */
            40U,	/* activeTh */
            10U,	/* activeHys */
            10U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            100U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
    },
    {   /* Power */
        {   
            /* Meta widget */
            &TSI_MetaWidgets[2],

            /* Widget enable status */
            TSI_WIDGET_ENABLE,

            /* Basic param */
            8U,	/* resolution */
            1U,	/* sampleClkDiv */
            0U,	/* idacMod */
            40U,	/* activeTh */
            10U,	/* activeHys */
            10U,	/* noiseTh */
            1U,	/* baselineIIRCoeff */
            {0UL, 0UL},	/* padding */
            3U,	/* onDebounce */
            1U,	/* offDebounce */
            100U,	/* sensitivity(used when param-only or full-auto calibration is enabled) */
            0U,	/* status */
        },
        0U,	/* buttonStatus */
    },
};

/* Mapping ----------------------------------------------------------------------*/
TSI_USED TSI_SensorTypeDef* const TSI_ChannelSensorMap[TSI_TOTAL_CHANNEL_NUM] TSI_CONST_CH_SNS_MAP_SECTION =
{
    /* Channel 0 */
    NULL,
    /* Channel 1 */
    NULL,
    /* Channel 2 */
    NULL,
    /* Channel 3 */
    NULL,
    /* Channel 4 */
    &TSI_SensorList.power[0],
    /* Channel 5 */
    &TSI_SensorList.slider[4],
    /* Channel 6 */
    &TSI_SensorList.slider[3],
    /* Channel 7 */
    &TSI_SensorList.slider[2],
    /* Channel 8 */
    &TSI_SensorList.slider[1],
    /* Channel 9 */
    &TSI_SensorList.slider[0],
    /* Channel 10 */
    &TSI_SensorList.mode[0],
    /* Channel 11 */
    NULL,
    /* Channel 12 */
    NULL,
    /* Channel 13 */
    NULL,
};

/* TSI widget pointers */
TSI_USED TSI_WidgetTypeDef* const TSI_WidgetPointers[TSI_TOTAL_WIDGETS_NUM] TSI_CONST_P_WIDGETS_SECTION =
{
    (TSI_WidgetTypeDef*)&TSI_WidgetList.mode,
    (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,
    (TSI_WidgetTypeDef*)&TSI_WidgetList.power,
};

/* TSI meta widgets */
TSI_USED TSI_MetaWidgetTypeDef const TSI_MetaWidgets[TSI_TOTAL_WIDGETS_NUM] TSI_CONST_META_WIDGETS_SECTION = 
{
    {   /* Mode */
        TSI_WIDGET_BUTTON,	/* type */
        TSI_SensorList.mode,	/* sensors */
        TSI_MODE_SENSORS_NUM,	/* sensorCnt */
    },
    {   /* Slider */
        TSI_WIDGET_SLIDER,	/* type */
        TSI_SensorList.slider,	/* sensors */
        TSI_SLIDER_SENSORS_NUM,	/* sensorCnt */
    },
    {   /* Power */
        TSI_WIDGET_BUTTON,	/* type */
        TSI_SensorList.power,	/* sensors */
        TSI_POWER_SENSORS_NUM,	/* sensorCnt */
    },
};

/* TSI meta sensors */
TSI_USED TSI_MetaSensorTypeDef const TSI_MetaSensors[TSI_TOTAL_SENSORS_NUM] TSI_CONST_META_SENSORS_SECTION = 
{
    {   /* Mode_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.mode,	/* parent */
        10,	/* channel */
        &TSI_FilterMode[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceMode[0],	/* debArray */
    },
    {   /* Slider_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        9,	/* channel */
        &TSI_FilterSlider[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[0],	/* debArray */
    },
    {   /* Slider_Sns1 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        8,	/* channel */
        &TSI_FilterSlider[1],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[1],	/* debArray */
    },
    {   /* Slider_Sns2 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        7,	/* channel */
        &TSI_FilterSlider[2],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[2],	/* debArray */
    },
    {   /* Slider_Sns3 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        6,	/* channel */
        &TSI_FilterSlider[3],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[3],	/* debArray */
    },
    {   /* Slider_Sns4 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.slider,	/* parent */
        5,	/* channel */
        &TSI_FilterSlider[4],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebounceSlider[4],	/* debArray */
    },
    {   /* Power_Sns0 */
        (TSI_WidgetTypeDef*)&TSI_WidgetList.power,	/* parent */
        4,	/* channel */
        &TSI_FilterPower[0],	/* filter */
        TSI_FILTER_REGULAR_SNS,	/* filterType */
        &TSI_DebouncePower[0],	/* debArray */
    },
};

/* TSI pins */
TSI_USED TSI_PinTypeDef const TSI_Pins[TSI_TOTAL_PINS_NUM] TSI_CONST_PINS_SECTION = 
{
    {   /* 0: Mode_Sns0 */
        10,
        0,
        10,
    },
    {   /* 1: Slider_Sns0 */
        10,
        4,
        9,
    },
    {   /* 2: Slider_Sns1 */
        9,
        4,
        8,
    },
    {   /* 3: Slider_Sns2 */
        7,
        3,
        7,
    },
    {   /* 4: Slider_Sns3 */
        4,
        2,
        6,
    },
    {   /* 5: Slider_Sns4 */
        3,
        2,
        5,
    },
    {   /* 6: Power_Sns0 */
        4,
        4,
        4,
    },
};

