/**
  *****************************************************************************
  * @file    main.c
  * @author  FMSH Application Team
  * @brief   Main program
  *****************************************************************************
  * @attention
  *
  * Copyright (c) [2021] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under Mulan PSL v2.
  * You can use this software according to the terms and conditions of the Mulan PSL v2.
  * You may obtain a copy of Mulan PSL v2 at:
  *          http://license.coscl.org.cn/MulanPSL2
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
  * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
  * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
  * See the Mulan PSL v2 for more details.
  *
  *****************************************************************************
  */

/* Private includes ---------------------------------------------------------*/
/* Library includes */
#include "tsi.h"
#include "tsi_widget.h"

/* FL driver library include */
#include "fm33fr0xx_fl.h"

/* USER PRIVATE INCLUDES BEGIN */
#include "led.h"
/* USER PRIVATE INCLUDES END */

/* Private variables --------------------------------------------------------*/
/* USER PRIVATE VARIABLES BEGIN */
uint8_t mode = 0U;
uint8_t modeOld = 1U;
uint8_t modeLock = 0U;

/* h = [0,360], s = [0,1], v=[0,1] */
float H=0, S=1, V=0.5f;  
/* USER PRIVATE VARIABLES END */

/* Private function prototypes ----------------------------------------------*/ 
static void SystemClockInit(void);

/* USER PRIVATE FUNCTION PROTOTYPES BEGIN */
static void DemoLed_ShowHSV(float h, float s, float v);
/* USER PRIVATE FUNCTION PROTOTYPES END */

/* Main ---------------------------------------------------------------------*/
int main(void)
{
    /* USER CODES AREA 1 BEGIN */

    /* USER CODES AREA 1 END */

    /* Init FL driver library */
    FL_Init();

    /* Init system clock */
    SystemClockInit();

    /* USER CODES AREA 2 BEGIN */
    
    /* DEMO Init ------------------------------------------------------------*/
    LED_Init();

    /* Demo LED */
    FL_GPIO_InitTypeDef demo_led_io = {0};
    demo_led_io.pin        = FL_GPIO_PIN_4 | FL_GPIO_PIN_5 | FL_GPIO_PIN_6;
	demo_led_io.mode       = FL_GPIO_MODE_DIGITAL;
	demo_led_io.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    demo_led_io.pull       = FL_GPIO_BOTH_DISABLE;
	demo_led_io.remapPin   = FL_GPIO_PINREMAP_FUNCTON3;	
    demo_led_io.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
	FL_GPIO_Init(GPIOB, &demo_led_io);
    
    // 使能TAU时钟
    FL_CMU_EnableGroup4BusClock(FL_CMU_GROUP4_BUSCLK_TAU0);
    FL_CMU_EnableGroup4BusClock(FL_CMU_GROUP4_BUSCLK_TAU1);

    // 配置输出比较模式
    FL_TAU_SetOperationMode(TAU01, FL_TAU_OPERATION_MODE_OC);
    FL_TAU_WritePrescaler(TAU01, 24 - 1);
    FL_TAU_WriteAutoReload(TAU01, 100 - 1);
    FL_TAU_WriteCompare(TAU01, 1 - 1);
    FL_TAU_OC_SetPolarity(TAU01, FL_TAU_OC_POLARITY_NORMAL);
    FL_TAU_EnableOutput(TAU01);
    
    FL_TAU_SetOperationMode(TAU03, FL_TAU_OPERATION_MODE_OC);
    FL_TAU_WritePrescaler(TAU03, 24 - 1);
    FL_TAU_WriteAutoReload(TAU03, 100 - 1);
    FL_TAU_WriteCompare(TAU03, 1 - 1);
    FL_TAU_OC_SetPolarity(TAU03, FL_TAU_OC_POLARITY_NORMAL);
    FL_TAU_EnableOutput(TAU03);
    
    FL_TAU_SetOperationMode(TAU11, FL_TAU_OPERATION_MODE_OC);
    FL_TAU_WritePrescaler(TAU11, 24 - 1);
    FL_TAU_WriteAutoReload(TAU11, 100 - 1);
    FL_TAU_WriteCompare(TAU11, 1 - 1);
    FL_TAU_OC_SetPolarity(TAU11, FL_TAU_OC_POLARITY_NORMAL);
    FL_TAU_EnableOutput(TAU11);
    
    FL_TAU_Enable(TAU01);
    FL_TAU_Enable(TAU03);
    FL_TAU_Enable(TAU11);
    
    // 1ms定时任务
    FL_TAU_SetOperationMode(TAU02, FL_TAU_OPERATION_MODE_NORMAL);
    FL_TAU_WritePrescaler(TAU02, 24 - 1);
    FL_TAU_WriteAutoReload(TAU02, 1000 - 1);
	FL_TAU_ClearFlag_Update(TAU02);
    FL_TAU_EnableIT_Update(TAU02);
	FL_CMU_EnableGroup2BusClock(FL_CMU_GROUP2_BUSCLK_INTMUX);
    INTMUX->CR |= 0x3U; // MUX0 选择 4
    NVIC_DisableIRQ(MUX0_IRQn);
    NVIC_SetPriority(MUX0_IRQn, 2);
    NVIC_EnableIRQ(MUX0_IRQn);
     
    // 清狗
    FL_IWDT_ReloadCounter(IWDT);
    
    /* USER CODES AREA 2 END */

    /* Init TSI */
    TSI_Init();

    /* Enable all widgets */
    TSI_Widget_EnableAll();

    /* Feed watchdog */
    FL_IWDT_ReloadCounter(IWDT);

    /* Start TSI */
    TSI_Start();

    /* Feed watchdog */
    FL_IWDT_ReloadCounter(IWDT);

    /* USER CODES AREA 3 BEGIN */

    /* USER CODES AREA 3 END */

    while(1)
    {   
        /* USER CODES AREA 4 BEGIN */
        uint8_t btnStat = 0U;
        uint8_t sldStat = 0U;
        int8_t firstActiveSeg = -1;
        int8_t firstInactiveSeg = -1;
        int i;
        /* USER CODES AREA 4 END */

        /* Feed watchdog */
        FL_IWDT_ReloadCounter(IWDT);
        
        /* USER CODES AREA 5 BEGIN */

        /* USER CODES AREA 5 END */
              
        /* Wait until a scan sequence has completed */
        if(TSI_GETSTAT_SCAN_CPLT())
        {   
            TSI_CLRSTAT_SCAN_CPLT();
            
            /* Update all widgets */
            TSI_Widget_UpdateAll();

            /* USER CODES AREA 6 BEGIN */
            /* 
                保护逻辑，当：
                - 仅按键大于等于2个同时按下
                - 仅滑条大于等于3段被按下
                - 仅滑条不相邻2段同时按下
                - 任意按键加上任意段滑条被按下
                禁止继续响应。
            */
            if(TSI_WidgetList.power.buttonStatus) btnStat++;
            if(TSI_WidgetList.mode.buttonStatus) btnStat++;
            for(i=0; i<TSI_WidgetList.slider.base.meta->sensorCnt; i++)
            {
                if(TSI_SensorList.slider[i].status != 0)
                {
                    sldStat++;
                    if(firstActiveSeg == -1)
                    {
                        firstActiveSeg = i;
                    }
                }
                else
                {
                    if(firstActiveSeg != -1 && firstInactiveSeg == -1)
                    {
                        firstInactiveSeg = i;
                    }
                }
            }
            
            if((btnStat >= 2 && sldStat == 0) || 
                (btnStat == 0 && sldStat > 3) || 
                (btnStat > 0 && sldStat > 0))
            {
                LED_Set(LED_GUARD);
                continue;
            }
            else if(firstInactiveSeg != -1)
            {
                if((firstInactiveSeg - firstActiveSeg) < sldStat)
                {
                    /* 不相邻Seg被按下 */
                    LED_Set(LED_GUARD);
                    continue;
                }
            }
            else
            {
                LED_Reset(LED_GUARD);
            }
            
            if(!TSI_WidgetList.power.buttonStatus && !TSI_WidgetList.mode.buttonStatus)
            {
                modeLock = 0;
            }
            
            if(TSI_WidgetList.mode.buttonStatus)
            {
                if(!modeLock)
                {
                    if(mode != 0)
                    {
                        if(mode == 1)
                        {
                            mode = 2;
                        }
                        else if(mode == 2)
                        {
                            mode = 1;
                        }
                        modeLock = 1;
                    }
                }
            }
            
            if(TSI_WidgetList.power.buttonStatus)
            {
                if(!modeLock)
                {
                    if(mode != 0)
                    {
                        modeOld = mode;
                        mode = 0;
                    }
                    else
                    {
                        mode = modeOld;
                    }
                    modeLock = 1;
                }
            }
            
            if(mode == 0)
            {
                LED_Reset(BTNMOD_LED);
                LED_Reset(BTNPWR_LED);
            }
            else if(mode == 1)
            {
                LED_Reset(BTNMOD_LED);
                LED_Set(BTNPWR_LED);
            }
            else if(mode == 2)
            {
                LED_Set(BTNMOD_LED);
                LED_Set(BTNPWR_LED);
            }
            
            if(TSI_WidgetList.slider.base.status)
            {
                int j;
                uint8_t center = TSI_WidgetList.slider.centerPos;
                
                /* 根据滑条值调节色彩 */
                if(mode == 1)
                {
                    H = center * 360 / 255;
                    
                    if(V == 0)
                    {
                        V = 1;
                    }
                    
                    if(S == 0)
                    {
                        S = 1;
                    }
                }
                else if(mode == 2)
                {
                    V = center / 255.0f;
                }
                
                for(j=0; j<TSI_WidgetList.slider.base.meta->sensorCnt; j++)
                {
                    if(TSI_SensorList.slider[j].status != 0)
                    {
                        switch(j)
                        {
                            case 0: LED_Set(SLD_LED1); break;
                            case 1: LED_Set(SLD_LED2); break;
                            case 2: LED_Set(SLD_LED3); break;
                            case 3: LED_Set(SLD_LED4); break;
                            case 4: LED_Set(SLD_LED5); break;
                        }
                    }
                    else
                    {
                        switch(j)
                        {
                            case 0: LED_Reset(SLD_LED1); break;
                            case 1: LED_Reset(SLD_LED2); break;
                            case 2: LED_Reset(SLD_LED3); break;
                            case 3: LED_Reset(SLD_LED4); break;
                            case 4: LED_Reset(SLD_LED5); break;
                        }
                    }
                }
            }
            else
            {
                LED_Reset(SLD_LED1);
                LED_Reset(SLD_LED2);
                LED_Reset(SLD_LED3);
                LED_Reset(SLD_LED4);
                LED_Reset(SLD_LED5);
            }
            
            if(mode == 0)
            {
                FL_TAU_WriteCompare(TAU01, 0);
                FL_TAU_WriteCompare(TAU11, 0);
                FL_TAU_WriteCompare(TAU03, 0);
            }
            else
            {
                DemoLed_ShowHSV(H, S, V);
            }
            /* USER CODES AREA 6 END */
        }

        /* USER CODES AREA 7 BEGIN */

        /* USER CODES AREA 7 END */
    }
}

/* Private functions --------------------------------------------------------*/
/** 
 * @brief Init system clock.
 * 
 */
void SystemClockInit(void)
{
    /* USER PRE SYSTEM CLOCK INIT BEGIN */

    /* USER PRE SYSTEM CLOCK INIT END */

    /* Enable RCHF 8MHz */
    FL_CMU_RCHF_WriteTrimValue(RCHF8M_TRIM);
    FL_CMU_RCHF_SetFrequency(FL_CMU_RCHF_FREQUENCY_8MHZ);

    /* Config PLL */
    FL_CMU_PLL_Disable();
    FL_CMU_PLL_SetClockSource(FL_CMU_PLL_CLK_SOURCE_RCHF);
    FL_CMU_PLL_SetPrescaler(FL_CMU_PLL_PSC_DIV8);
    FL_CMU_PLL_WriteMultiplier(48 - 1);
    FL_CMU_PLL_SetOutputMultiplier(FL_CMU_PLL_OUTPUT_X1);
    FL_CMU_PLL_Enable();
    
    /* Wait for the PLL lock flag */
    uint32_t timeout = 0xFFFFFFFFUL;
    do
    {
        if(FL_CMU_IsActiveFlag_PLLReady() == 0x1U) 
        {
            break;
        }

        /* Clear watchdog */
        FL_IWDT_ReloadCounter(IWDT);

    } while (--timeout > 0);

    /* Set flash read wait */
    FL_FLASH_SetCodeReadWait(FLASH, FL_FLASH_READ_CODEWAIT_1CYCLE);

    /* Set system clock source and bus prescaler */
    FL_CMU_SetAHBPrescaler(FL_CMU_AHBCLK_PSC_DIV2);
    FL_CMU_SetAPBPrescaler(FL_CMU_APBCLK_PSC_DIV1);
    FL_CMU_SetSystemClockSource(FL_CMU_SYSTEM_CLK_SOURCE_PLL);

    /* Update system core clock */
    SystemCoreClock = 24000000;

    /* USER SYSTEM CLOCK INIT BEGIN */

    /* USER SYSTEM CLOCK INIT END */
}

/* USER PRIVATE FUNCTION BEGIN */
void DemoLed_ShowHSV(float h, float s, float v)
{
    float a,b,c;
    uint8_t R,G,B;
    uint8_t i;
    float f;

    if(s == 0)
    {
        R=G=B=v;
    }
    else
    {
        h /= 60;
        i = (uint8_t)h;
        f = h - i;
        a = v * (1-s);
        b = v * (1-s*f);
        c = v * (1-s*(1-f));
        switch(i)
        {
            case 0:
                R=v*255; G=c*255; B=a*255;
                break;
            case 1:
                R=b*255; G=v*255; B=a*255;
                break;
            case 2:
                R=a*255; G=v*255; B=c*255;
                break;
            case 3:
                R=a*255; G=b*255; B=v*255;
                break;
            case 4:
                R=c*255; G=a*255; B=v*255;
                break;
            case 5:
                R=v*255; G=a*255; B=b*255;
                break;
        }
    }

    // 调节Demo LED
    FL_TAU_WriteCompare(TAU01, G * 100 / 255);
    FL_TAU_WriteCompare(TAU11, B * 100 / 255);
    FL_TAU_WriteCompare(TAU03, R * 100 / 255);    
}

void MUX0_IRQHandler(void)
{
    static int cnt = 0;
    static int stat = 0;
    
    if(FL_TAU_IsEnabledIT_Update(TAU02) && FL_TAU_IsActiveFlag_Update(TAU02))
    {
        FL_TAU_ClearFlag_Update(TAU02);
        cnt++;
        if(stat == 0 && cnt > 1000)
        {
            cnt = 0;
            stat = 1;
            LED_Set(LED_RUN);
        }
        else if(stat == 1 && cnt > 100)
        {
            cnt = 0;
            stat = 0;
            LED_Reset(LED_RUN);
        }
    }
}
/* USER PRIVATE FUNCTION END */

